@@ -1,17 +1,27 @@
 mod of_sexp;
 mod parse;
+mod sexp_of;
 
 pub use of_sexp::*;
 pub use parse::*;
+pub use sexp_of::*;
 use std::io::Write;
 
 /// Type for S-expressions using owned values.
-#[derive(Debug, PartialEq, Eq, Clone)]
+#[derive(Debug, PartialEq, Eq, Clone, Hash)]
 pub enum Sexp {
     Atom(Vec<u8>),
     List(Vec<Sexp>),
 }
 
+pub fn atom(atom: &[u8]) -> Sexp {
+    Sexp::Atom(atom.to_vec())
+}
+
+pub fn list(list: &[Sexp]) -> Sexp {
+    Sexp::List(list.to_vec())
+}
+
 // This trait is used to mark types for which using the to/from string
 // conversion is fine.
 pub trait UseToString {}
@@ -33,80 +43,6 @@ impl UseToString for f64 {}
 impl UseToString for f32 {}
 impl UseToString for bool {}
 
-impl<T: ToString + UseToString> From<T> for Sexp {
-    fn from(t: T) -> Self {
-        Sexp::Atom(t.to_string().as_bytes().to_vec())
-    }
-}
-
-impl From<&str> for Sexp {
-    fn from(s: &str) -> Self {
-        Sexp::Atom(s.as_bytes().to_vec())
-    }
-}
-
-impl<'a> From<&BytesSlice<'a>> for Sexp {
-    fn from(s: &BytesSlice<'a>) -> Self {
-        Sexp::Atom(s.0.to_vec())
-    }
-}
-
-impl From<String> for Sexp {
-    fn from(s: String) -> Self {
-        Sexp::Atom(s.as_bytes().to_vec())
-    }
-}
-
-impl<'a, T: 'a> From<&'a [T]> for Sexp
-where
-    &'a T: Into<Sexp>,
-{
-    fn from(t: &'a [T]) -> Self {
-        Sexp::List(t.iter().map(|x| x.into()).collect())
-    }
-}
-
-impl<T1, T2> From<(T1, T2)> for Sexp
-where
-    T1: Into<Sexp>,
-    T2: Into<Sexp>,
-{
-    fn from(t: (T1, T2)) -> Self {
-        Sexp::List(vec![t.0.into(), t.1.into()])
-    }
-}
-
-impl<T1, T2, T3> From<(T1, T2, T3)> for Sexp
-where
-    T1: Into<Sexp>,
-    T2: Into<Sexp>,
-    T3: Into<Sexp>,
-{
-    fn from(t: (T1, T2, T3)) -> Self {
-        Sexp::List(vec![t.0.into(), t.1.into(), t.2.into()])
-    }
-}
-
-impl<'a, K, V> From<&'a std::collections::HashMap<K, V>> for Sexp
-where
-    &'a K: Into<Sexp>,
-    &'a V: Into<Sexp>,
-{
-    fn from(map: &'a std::collections::HashMap<K, V>) -> Self {
-        Sexp::List(map.iter().map(|(k, v)| (k, v).into()).collect())
-    }
-}
-
-impl<'a, K, V> From<&'a std::collections::BTreeMap<K, V>> for Sexp
-where
-    &'a K: Into<Sexp>,
-    &'a V: Into<Sexp>,
-{
-    fn from(map: &'a std::collections::BTreeMap<K, V>) -> Self {
-        Sexp::List(map.iter().map(|(k, v)| (k, v).into()).collect())
-    }
-}
-
 // Serialization
 
 fn must_escape(data: &[u8]) -> bool {
